( cd usr/lib ; rm -rf libtunepimp.so.3 )
( cd usr/lib ; ln -sf libtunepimp.so.3.0.0 libtunepimp.so.3 )
( cd usr/lib ; rm -rf libtunepimp.so )
( cd usr/lib ; ln -sf libtunepimp.so.3.0.0 libtunepimp.so )
